/// <reference types="jest" />
import { Logger } from 'bs-logger';
import { CustomTransformers, Diagnostic, ParsedCommandLine } from 'typescript';
import { AstTransformerDesc, BabelConfig, BabelJestTransformer, TTypeScript, TsCompiler, TsJestConfig, TsJestGlobalOptions, TsJestHooksMap } from '../types';
import { JsonableValue } from '../util/jsonable-value';
import { TSError } from '../util/ts-error';
interface ReadTsConfigResult {
    input?: any;
    resolved: ParsedCommandLine;
}
export declare const MATCH_NOTHING: RegExp;
export declare const IGNORE_DIAGNOSTIC_CODES: number[];
export declare class ConfigSet {
    private readonly _jestConfig;
    readonly parentOptions?: TsJestGlobalOptions | undefined;
    readonly logger: Logger;
    constructor(_jestConfig: jest.ProjectConfig, parentOptions?: TsJestGlobalOptions | undefined, parentLogger?: Logger);
    readonly jest: jest.ProjectConfig;
    readonly tsJest: TsJestConfig;
    readonly typescript: ParsedCommandLine;
    readonly tsconfig: any;
    readonly versions: Record<string, string>;
    private readonly _typescript;
    readonly raiseDiagnostics: (diagnostics: Diagnostic[], filePath?: string | undefined, logger?: Logger) => void;
    readonly babel: BabelConfig | undefined;
    readonly compilerModule: TTypeScript;
    readonly babelJestTransformer: BabelJestTransformer | undefined;
    readonly tsCompiler: TsCompiler;
    readonly astTransformers: AstTransformerDesc[];
    readonly tsCustomTransformers: CustomTransformers;
    readonly hooks: TsJestHooksMap;
    readonly filterDiagnostics: (diagnostics: Diagnostic[], filePath?: string | undefined) => Diagnostic[];
    readonly shouldReportDiagnostic: (filePath: string) => boolean;
    readonly shouldStringifyContent: (filePath: string) => boolean;
    readonly createTsError: (diagnostics: ReadonlyArray<Diagnostic>) => TSError;
    readonly tsCacheDir: string | undefined;
    readonly rootDir: string;
    readonly cwd: string;
    readonly isDoctoring: boolean;
    readTsConfig(compilerOptions?: object, resolvedConfigFile?: string | null, noProject?: boolean | null): ReadTsConfigResult;
    resolvePath(inputPath: string, { throwIfMissing, nodeResolve }?: {
        throwIfMissing?: boolean;
        nodeResolve?: boolean;
    }): string;
    readonly jsonValue: JsonableValue<{
        versions: Record<string, string>;
        transformers: string[];
        jest: {
            automock: boolean;
            browser: boolean;
            cache: boolean;
            cacheDirectory: string;
            clearMocks: boolean;
            coveragePathIgnorePatterns: string[];
            cwd: string;
            detectLeaks: boolean;
            displayName: jest.Maybe<string>;
            forceCoverageMatch: string[];
            globals: object;
            haste: jest.HasteConfig;
            moduleDirectories: string[];
            moduleFileExtensions: string[];
            moduleLoader: string;
            moduleNameMapper: [string, string][];
            modulePathIgnorePatterns: string[];
            modulePaths: string[];
            name: string;
            resetMocks: boolean;
            resetModules: boolean;
            resolver: jest.Maybe<string>;
            rootDir: string;
            roots: string[];
            runner: string;
            setupFiles: string[];
            setupTestFrameworkScriptFile: string;
            skipNodeResolution: boolean;
            snapshotSerializers: string[];
            testEnvironment: string;
            testEnvironmentOptions: object;
            testLocationInResults: boolean;
            testMatch: string[];
            testPathIgnorePatterns: string[];
            testRegex: string;
            testRunner: string;
            testURL: string;
            timers: "real" | "fake";
            transform: [string, string][];
            transformIgnorePatterns: string[];
            unmockedModulePathPatterns: jest.Maybe<string[]>;
            watchPathIgnorePatterns: string[];
        };
        tsJest: TsJestConfig;
        babel: import("babel__core").TransformOptions | undefined;
        tsconfig: any;
    }>;
    readonly cacheKey: string;
    toJSON(): {
        versions: Record<string, string>;
        transformers: string[];
        jest: {
            automock: boolean;
            browser: boolean;
            cache: boolean;
            cacheDirectory: string;
            clearMocks: boolean;
            coveragePathIgnorePatterns: string[];
            cwd: string;
            detectLeaks: boolean;
            displayName: jest.Maybe<string>;
            forceCoverageMatch: string[];
            globals: object;
            haste: jest.HasteConfig;
            moduleDirectories: string[];
            moduleFileExtensions: string[];
            moduleLoader: string;
            moduleNameMapper: [string, string][];
            modulePathIgnorePatterns: string[];
            modulePaths: string[];
            name: string;
            resetMocks: boolean;
            resetModules: boolean;
            resolver: jest.Maybe<string>;
            rootDir: string;
            roots: string[];
            runner: string;
            setupFiles: string[];
            setupTestFrameworkScriptFile: string;
            skipNodeResolution: boolean;
            snapshotSerializers: string[];
            testEnvironment: string;
            testEnvironmentOptions: object;
            testLocationInResults: boolean;
            testMatch: string[];
            testPathIgnorePatterns: string[];
            testRegex: string;
            testRunner: string;
            testURL: string;
            timers: "real" | "fake";
            transform: [string, string][];
            transformIgnorePatterns: string[];
            unmockedModulePathPatterns: jest.Maybe<string[]>;
            watchPathIgnorePatterns: string[];
        };
        tsJest: TsJestConfig;
        babel: import("babel__core").TransformOptions | undefined;
        tsconfig: any;
    };
}
export {};
